@@ -71,6 +71,7 @@ typedef enum {
     S2N_ERR_ENCRYPT = S2N_ERR_T_PROTO_START,
     S2N_ERR_DECRYPT,
     S2N_ERR_BAD_MESSAGE,
+    S2N_ERR_UNEXPECTED_CERT_REQUEST,
     S2N_ERR_KEY_INIT,
     S2N_ERR_KEY_DESTROY,
     S2N_ERR_DH_SERIALIZING,
@@ -321,6 +322,7 @@ typedef enum {
     S2N_ERR_KTLS_RENEG,
     S2N_ERR_ATOMIC,
     S2N_ERR_KTLS_KEY_LIMIT,
+    S2N_ERR_SECURITY_POLICY_INCOMPATIBLE_CERT,
     S2N_ERR_T_USAGE_END,
 } s2n_error;
 
