@@ -48,6 +48,10 @@
 #define NUMBER_OF_RANGE_FUNCTION_CALLS 200
 #define MAX_REPEATED_OUTPUT            4
 
+S2N_RESULT s2n_rand_device_validate(struct s2n_rand_device *device);
+S2N_RESULT s2n_rand_get_urandom_for_test(struct s2n_rand_device **device);
+S2N_RESULT s2n_rand_set_urandom_for_test();
+
 struct random_test_case {
     const char *test_case_label;
     int (*test_case_cb)(struct random_test_case *test_case);
@@ -788,6 +792,87 @@ static int s2n_random_rand_bytes_after_cleanup_cb(struct random_test_case *test_
     return S2N_SUCCESS;
 }
 
+static int s2n_random_rand_bytes_before_init(struct random_test_case *test_case)
+{
+#if S2N_LIBCRYPTO_SUPPORTS_CUSTOM_RAND
+    /* Calling RAND_bytes will set a global random method */
+    unsigned char rndbytes[16] = { 0 };
+    EXPECT_EQUAL(RAND_bytes(rndbytes, sizeof(rndbytes)), 1);
+    const RAND_METHOD *rand_method = RAND_get_rand_method();
+    EXPECT_NOT_NULL(rand_method);
+    EXPECT_NOT_EQUAL(rand_method->bytes, s2n_openssl_compat_rand);
+
+    EXPECT_SUCCESS(s2n_init());
+
+    /* The global random method is overridden after calling s2n_init() */
+    const RAND_METHOD *custom_rand_method = RAND_get_rand_method();
+    EXPECT_NOT_NULL(custom_rand_method);
+    EXPECT_EQUAL(custom_rand_method->bytes, s2n_openssl_compat_rand);
+
+    /* RAND_bytes is still successful */
+    EXPECT_EQUAL(RAND_bytes(rndbytes, sizeof(rndbytes)), 1);
+
+#endif
+    return S2N_SUCCESS;
+}
+
+static int s2n_random_invalid_urandom_fd_cb(struct random_test_case *test_case)
+{
+    EXPECT_SUCCESS(s2n_disable_atexit());
+
+    struct s2n_rand_device *dev_urandom = NULL;
+    EXPECT_OK(s2n_rand_get_urandom_for_test(&dev_urandom));
+    EXPECT_NOT_NULL(dev_urandom);
+
+    for (size_t test = 0; test <= 1; test++) {
+        EXPECT_EQUAL(dev_urandom->fd, -1);
+
+        /* Validation should fail before initialization. */
+        EXPECT_ERROR(s2n_rand_device_validate(dev_urandom));
+
+        EXPECT_SUCCESS(s2n_init());
+
+        /* Validation should succeed after initialization. */
+        EXPECT_OK(s2n_rand_device_validate(dev_urandom));
+
+        /* Override the mix callback with urandom, in case support for rdrand is detected and enabled. */
+        EXPECT_OK(s2n_rand_set_urandom_for_test());
+
+        EXPECT_TRUE(dev_urandom->fd > STDERR_FILENO);
+        if (test == 0) {
+            /* Close the file descriptor. */
+            EXPECT_EQUAL(close(dev_urandom->fd), 0);
+        } else {
+            /* Make the file descriptor invalid by pointing it to STDERR. */
+            dev_urandom->fd = STDERR_FILENO;
+        }
+
+        /* Validation should fail when the file descriptor is invalid. */
+        EXPECT_ERROR(s2n_rand_device_validate(dev_urandom));
+
+        s2n_stack_blob(rand_data, 16, 16);
+        EXPECT_OK(s2n_get_public_random_data(&rand_data));
+
+        uint64_t public_bytes_used = 0;
+        EXPECT_OK(s2n_get_public_random_bytes_used(&public_bytes_used));
+
+        if (s2n_is_in_fips_mode()) {
+            /* The urandom implementation should not be in use when s2n-tls is in FIPS mode. */
+            EXPECT_EQUAL(public_bytes_used, 0);
+        } else {
+            /* When the urandom implementation is used, the file descriptor is re-opened and
+             * validation should succeed.
+             */
+            EXPECT_OK(s2n_rand_device_validate(dev_urandom));
+            EXPECT_TRUE(public_bytes_used > 0);
+        }
+
+        EXPECT_SUCCESS(s2n_cleanup());
+    }
+
+    return S2N_SUCCESS;
+}
+
 struct random_test_case random_test_cases[] = {
     { "Random API.", s2n_random_test_case_default_cb, CLONE_TEST_DETERMINE_AT_RUNTIME, EXIT_SUCCESS },
     { "Random API without prediction resistance.", s2n_random_test_case_without_pr_cb, CLONE_TEST_DETERMINE_AT_RUNTIME, EXIT_SUCCESS },
@@ -800,6 +885,8 @@ struct random_test_case random_test_cases[] = {
      */
     { "Test failure.", s2n_random_test_case_failure_cb, CLONE_TEST_DETERMINE_AT_RUNTIME, 1 },
     { "Test libcrypto's RAND engine is reset correctly after manual s2n_cleanup()", s2n_random_rand_bytes_after_cleanup_cb, CLONE_TEST_DETERMINE_AT_RUNTIME, EXIT_SUCCESS },
+    { "Test getting entropy with an invalid file descriptor", s2n_random_invalid_urandom_fd_cb, CLONE_TEST_DETERMINE_AT_RUNTIME, EXIT_SUCCESS },
+    { "Test libcrypto's global RAND is unset after calling s2n_init()", s2n_random_rand_bytes_before_init, CLONE_TEST_DETERMINE_AT_RUNTIME, EXIT_SUCCESS },
 };
 
 int main(int argc, char **argv)
